@@ -25,17 +25,17 @@ using namespace Rcpp;
 //' @export
 //' @keywords internal
 //[[Rcpp::export]]
-Rcpp::NumericMatrix popdynOneTScpp(double nareas, double maxage, Rcpp::NumericVector SSBcurr,
-                                Rcpp::NumericMatrix Ncurr,  Rcpp::NumericMatrix Zcurr, double PerrYr,
-                                double hs,  Rcpp::NumericVector R0a,  Rcpp::NumericVector SSBpR,
-                                Rcpp::NumericVector aR,  Rcpp::NumericVector bR,  Rcpp::NumericMatrix mov,
-                                double SRrel) {
+arma::mat popdynOneTScpp(double nareas, double maxage, Rcpp::NumericVector SSBcurr,
+                         NumericMatrix Ncurr,  Rcpp::NumericMatrix Zcurr, double PerrYr,
+                         double hs,  Rcpp::NumericVector R0a,  Rcpp::NumericVector SSBpR,
+                         Rcpp::NumericVector aR,  Rcpp::NumericVector bR,  arma::cube mov,
+                         double SRrel) {
   
-  Rcpp::NumericMatrix Nnext(maxage, nareas);
-  Rcpp::NumericMatrix tempMat2(nareas, nareas);	
-  Rcpp::NumericMatrix tempMat3(nareas, nareas);
-  Rcpp::NumericMatrix Nstore(maxage, nareas); 
+  arma::mat Nnext(maxage, nareas);
+  // arma::mat tempMat2(nareas, nareas);	
+  arma::mat Nstore(maxage, nareas); 
 
+  
   // Recruitment assuming regional R0 and stock wide steepness
   for (int A=0; A < nareas; A++) {
     if (SRrel == 1) {
@@ -47,23 +47,28 @@ Rcpp::NumericMatrix popdynOneTScpp(double nareas, double maxage, Rcpp::NumericVe
       
       Nnext(0, A) = PerrYr * aR(A) * SSBcurr(A) * exp(-bR(A) * SSBcurr(A));
     }
-    
     // Mortality
     for (int age=1; age<maxage; age++) {
       Nnext(age, A) = Ncurr(age-1, A) * exp(-Zcurr(age-1, A)); // Total mortality
     }
   }
-
+  // 
+  
   // Move stock
   for (int age=0; age<maxage; age++) {
-    for (int AA = 0; AA < nareas; AA++) {
-      for (int BB = 0; BB < nareas; BB++) {
-        if (AA != BB) tempMat2(AA, BB) = (Nnext(age, AA) * mov(AA, AA)) +   (Nnext(age, BB) * mov(BB, AA));
-        tempMat3(AA, BB) = tempMat2(AA, BB);				 
+  
+    arma::mat tempMat2(nareas, nareas);
+    for (int AA = 0; AA < nareas; AA++) {   // (from areas)
+      for (int BB = 0; BB < nareas; BB++) { // (to areas)
+        arma::vec temp = mov.subcube(age, AA, BB, age, AA, BB);
+        double temp2 = temp(0);
+        tempMat2(BB, AA) = Nnext(age, AA) * temp2; // movement fractions
       }
-      Nstore(age, AA) = sum(tempMat2.row(AA));
     }
-  }   
+    for (int BB = 0; BB < nareas; BB++) { // to areas
+      Nstore(age, BB) = sum(tempMat2.row(BB));   // sums all rows (from areas) in each column (to areas)
+    }
+  }
   
   return Nstore;
 } 
@@ -99,20 +104,21 @@ Rcpp::NumericMatrix popdynOneTScpp(double nareas, double maxage, Rcpp::NumericVe
 //' @param Qc Numeric. Catchability coefficient
 //' @param Fapic Numeric. Apical F value
 //' @param maxF A numeric value specifying the maximum fishing mortality for any single age class
+//' @param MPA Spatial closure by year and area
 //' @param control Integer. 1 to use q and effort to calculate F, 2 to use Fapic (apical F) and 
 //' vulnerablity to calculate F.
 //' 
 //' @author A. Hordyk
 //' 
-//' @export
 //' @keywords internal
 //[[Rcpp::export]]
 List popdynCPP(double nareas, double maxage, arma::mat Ncurr, double pyears,
                arma::mat M_age, arma::vec Asize_c, arma::mat MatAge, arma::mat WtAge,
                arma::mat Vuln, arma::mat Retc, arma::vec Prec,
-               NumericMatrix movc, double SRrelc, arma::vec Effind,
+               arma::cube movc, double SRrelc, arma::vec Effind,
                double Spat_targc, double hc, NumericVector R0c, NumericVector SSBpRc,
-               NumericVector aRc, NumericVector bRc, double Qc, double Fapic, double maxF, int control) {
+               NumericVector aRc, NumericVector bRc, double Qc, double Fapic, double maxF, 
+               arma::mat MPA, int control, double SSB0c) {
   
   arma::cube Narray(maxage, pyears, nareas, arma::fill::zeros);
   arma::cube Barray(maxage, pyears, nareas, arma::fill::zeros);
@@ -127,8 +133,18 @@ List popdynCPP(double nareas, double maxage, arma::mat Ncurr, double pyears,
   NumericVector tempVec(nareas);
   arma::vec fishdist(nareas);
   
+  // make clones of spawning/recruit by area because they are updated (sometimes)
+  NumericVector R0c2(clone(R0c));
+  NumericVector aRc2(clone(aRc));
+  NumericVector bRc2(clone(bRc));
+  
+  NumericVector SSB0a(nareas);
+  double R0 = sum(R0c);
+
+
   // Initial year
   Narray.subcube(0, 0, 0, maxage-1, 0, nareas-1) = Ncurr;
+  
   for (int A=0; A<nareas; A++) {
     Barray.subcube(0, 0, A, maxage-1, 0, A) = Ncurr.col(A) % WtAge.col(0);
     SSNarray.subcube(0, 0, A, maxage-1, 0, A) = Ncurr.col(A) % MatAge.col(0); 
@@ -138,6 +154,7 @@ List popdynCPP(double nareas, double maxage, arma::mat Ncurr, double pyears,
     tempVec(A) = accu(VBarray.slice(A));
   }
   
+
   // fishdist = (pow(tempVec, Spat_targc))/mean((pow(tempVec, Spat_targc)));
   fishdist = (pow(tempVec, Spat_targc))/sum((pow(tempVec, Spat_targc)));
 
@@ -155,6 +172,8 @@ List popdynCPP(double nareas, double maxage, arma::mat Ncurr, double pyears,
     }
   }
   
+ 
+  
   // apply Fmax condition 
   arma::uvec tempvals = arma::find(FMarray > (1-exp(-maxF)));
   FMarray.elem(tempvals).fill(1-exp(-maxF));
@@ -163,17 +182,21 @@ List popdynCPP(double nareas, double maxage, arma::mat Ncurr, double pyears,
   
   Zarray.subcube(0,0, 0, maxage-1, 0, nareas-1) = Marray.subcube(0,0, 0, maxage-1, 0, nareas-1) + FMarray.subcube(0,0, 0, maxage-1, 0, nareas-1);
   
-  
+
   for (int yr=0; yr<(pyears-1); yr++) {
-    
+    // Rcpp::Rcout << "yr = " << yr << std::endl;
     arma::vec SB(nareas);
     
     for (int A=0; A<nareas; A++) SB(A) = accu(SBarray.subcube(0, yr, A, maxage-1, yr, A));
+    if ((yr >0) & (control==3)) SB = SSB0a;
+          
+          
     arma::mat Ncurr2 = Narray.subcube(0, yr, 0, maxage-1, yr, nareas-1);
     arma::mat Zcurr = Zarray.subcube(0, yr, 0, maxage-1, yr, nareas-1);
-    NumericMatrix NextYrNa = popdynOneTScpp(nareas, maxage, wrap(SB), wrap(Ncurr2), wrap(Zcurr), 
-                                       Prec(yr+1+maxage), hc, R0c, SSBpRc, aRc, bRc, movc, SRrelc); 
-    arma::mat NextYrN = as<arma::mat>(NextYrNa);
+    
+    arma::mat NextYrN = popdynOneTScpp(nareas, maxage, wrap(SB), wrap(Ncurr2), wrap(Zcurr), 
+                                       Prec(yr+1+maxage), hc, R0c2, SSBpRc, aRc2, bRc2, movc, SRrelc); 
+  
 
     Narray.subcube(0, yr+1, 0, maxage-1, yr+1, nareas-1) = NextYrN;
     for (int A=0; A<nareas; A++) {
@@ -184,9 +207,20 @@ List popdynCPP(double nareas, double maxage, arma::mat Ncurr, double pyears,
       Marray.subcube(0, yr+1, A, maxage-1, yr+1, A) = M_age.col(yr+1);
       tempVec(A) = accu(VBarray.subcube(0, yr+1, A, maxage-1, yr+1, A));
     }
-
+  
     // fishdist = (pow(tempVec, Spat_targc))/mean((pow(tempVec, Spat_targc)));
     fishdist = (pow(tempVec, Spat_targc))/sum((pow(tempVec, Spat_targc)));
+    
+    arma::vec d1(nareas);
+    for (int A=0; A<nareas; A++) {
+      d1(A) = MPA(yr,A) * fishdist(A);// historical closures
+    }
+    double fracE = sum(d1); // fraction of current effort in open areas
+    arma::vec fracE2(nareas);
+    for (int A=0; A<nareas; A++) {
+      fracE2(A) = d1(A) * (fracE + (1-fracE))/fracE;
+    }
+    fishdist = fracE2;
 
     // calculate F at age for next year
     if (control == 1) {
@@ -195,20 +229,42 @@ List popdynCPP(double nareas, double maxage, arma::mat Ncurr, double pyears,
         FMretarray.subcube(0,yr+1, A, maxage-1, yr+1, A) =  (Effind(yr+1) * Qc * fishdist(A) * Retc.col(yr+1))/Asize_c(A);
       }
     }
+
     if (control == 2) {
       for (int A=0; A<nareas; A++) {
         FMarray.subcube(0,yr+1, A, maxage-1, yr+1, A) =  (Fapic * fishdist(A) * Vuln.col(yr+1))/Asize_c(A);
         FMretarray.subcube(0,yr+1, A, maxage-1, yr+1, A) =  (Fapic * fishdist(A) * Retc.col(yr+1))/Asize_c(A);
       }
     }
-    // apply Fmax condition 
-    arma::uvec tempvals3 = arma::find(FMarray > (1-exp(-maxF)));
-    FMarray.elem(tempvals3).fill(1-exp(-maxF));
-    arma::uvec tempvals4 = arma::find(FMretarray > (1-exp(-maxF)));
-    FMretarray.elem(tempvals4).fill(1-exp(-maxF));
     
-    Zarray.subcube(0,yr+1, 0, maxage-1, yr+1, nareas-1) = Marray.subcube(0,yr+1, 0, maxage-1, yr+1, nareas-1) + FMarray.subcube(0,yr+1, 0, maxage-1, yr+1, nareas-1);
+    if (control ==3) { // simulate unfished dynamics & update recruitment by area
+      Zarray.subcube(0,yr+1, 0, maxage-1, yr+1, nareas-1) = Marray.subcube(0,yr+1, 0, maxage-1, yr+1, nareas-1);
+      
+      // get spatial distribution 
+      for (int A=0; A<nareas; A++) {
+        SSB0a(A) = accu(SBarray.subcube(0, yr+1, A, maxage-1, yr+1, A));
+        R0c2(A) = accu(SBarray.subcube(0, yr, A, maxage-1, yr, A));
+      }
+      
+      SSB0a =   SSB0a/ (sum(SSB0a)/SSB0c); 
+      R0c2 = R0c2/ (sum(R0c2)/R0);
+      
+      // recalculate recruitment parameters
+      for (int A=0; A<nareas; A++) {
+        bRc2(A) = log(5 * hc)/(0.8 * SSB0a(A));
+        aRc2(A) = exp(bRc2(A) * SSB0a(A))/ SSBpRc(A);
+      }
 
+    } else {
+      // apply Fmax condition 
+      arma::uvec tempvals3 = arma::find(FMarray > (1-exp(-maxF)));
+      FMarray.elem(tempvals3).fill(1-exp(-maxF));
+      arma::uvec tempvals4 = arma::find(FMretarray > (1-exp(-maxF)));
+      FMretarray.elem(tempvals4).fill(1-exp(-maxF));
+      Zarray.subcube(0,yr+1, 0, maxage-1, yr+1, nareas-1) = Marray.subcube(0,yr+1, 0, maxage-1, yr+1, nareas-1) + FMarray.subcube(0,yr+1, 0, maxage-1, yr+1, nareas-1);
+      
+    }
+   
   }
 
   List out(8);
